/************************************************************
;                                                           *
;  William M. Spears					    *
;  Navy Center for Applied Research in AI                   *
;  Naval Research Laboratory                                *
;                                                           *
;  This software is the property of the Department of the   *
;  Navy. Permission is hereby granted to copy all or any    *
;  part of this program for free distribution, however      *
;  this header is required on all copies.		    *
;                                                           *
; File: cross.c                                             *
;************************************************************/

#include "header.h"

int cross_change;

/* Swap one bit position in two individuals.*/

void cross_swap (int i, int j, int x)
{
  int tempi, tempj;
  
  tempi = c[i][x];
  tempj = c[j][x];
  
  if (tempi != tempj) {
    cross_change = 1;
    c[i][x] = tempj;
    c[j][x] = tempi;
  }

  return;
}

/* Either One or Two point cross-over. Mark for re-evaluation if
   a change has occurred.. */

void cross_over (int i, int j, int x1, int y1)
{
  int x;

  for (x = 1 + x1; x <= y1; x++) cross_swap(i, j, x);

  return;
}

/* Do cross-over of two individuals. The following diagram should help.

 		1	2	3					*B*
 	+---------------------------------------------------------------+
  i:	|   1	|   2	|   3	|	|	|	|	|  *B*	|
 	+---------------------------------------------------------------+

 		1	2	3					*B*
 	+---------------------------------------------------------------+
  j:	|   1	|   2	|   3	|	|	|	|	|  *B*	|
 	+---------------------------------------------------------------+

*/

/* Standard n-point crossover, n is the number of crossover points. */

void cross (int n, int i, int j)
{
  int x, y, temp;
  int cross_array[MAX_BITS];
  
  /* Fill in crossover points into an array. */
  
  for (temp = 0; temp < n; temp++) {
    cross_array[temp] = myrand(length);
  }
  
  /* If odd number of crossover points, make an even number
     by using the length as the last crossover point. */
  
  if (n % 2) {
    n++;
    cross_array[n - 1] = length;
  }
  
  /* Sort the array. */
  
  selectsort(n, cross_array);
  
  /* Perform crossover over the right sections. */
  
  cross_change = 0;
  
  for (temp = 0; temp < n; temp = temp + 2) {
    x = cross_array[temp];
    y = cross_array[temp+1];
    if (x != y) cross_over(i, j, x, y);
  }
  
  if (cross_change) {
    reevaluations = reevaluations + 2;
    c_value[i] = -1.0;		/* re_evaluate */
    c_value[j] = -1.0;		/* re_evaluate */
  }

  return;
}

/* Cross-over a percentage of the population, based on the
   cross-over rate and the population size. The number of
   crossover points (n) should be less than the length of
   the chromosome (string). */

void cross_population (void)
{
  int i;
  
  reevaluations = 0;
  for (i = 1; i <= (CROSS_OVER * size); i = i + 2) {
    cross(cross_points, i, i + 1);
  }

  return;
}

/* Standard uniform crossover with .5 probability of swapping
   alleles. Could be generalized to arbitrary probability. */

void uniform_cross (int i, int j)
{
  int x;
  
  cross_change = 0;
  
  for (x = 1; x <= length; x++) {
    if (brand()) cross_swap(i, j, x);
  }
  
  if (cross_change) {
    reevaluations = reevaluations + 2;
    c_value[i] = -1.0;		/* re_evaluate */
    c_value[j] = -1.0;		/* re_evaluate */
  }

  return;
}

/* Apply uniform crossover to a percentage of the population. */

void uniform_cross_population (void)
{
  int i;
  
  reevaluations = 0;
  
  for (i = 1; i <= (CROSS_OVER * size); i = i + 2) {
    uniform_cross(i, i + 1);
  }

  return;
}
